# -*- coding: utf-8 -*-
from distutils import dir_util
import os
import pytest
from jatsutils import JatsElementTree as JET


@pytest.fixture()
def datadir(tmpdir, request):
    '''
    Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    '''
    filename = request.module.__file__
    test_dir, _ = os.path.splitext(filename)

    if os.path.isdir(test_dir):
        dir_util.copy_tree(test_dir, bytes(tmpdir))

    return tmpdir


@pytest.fixture()
def static_dir():
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), "static")


@pytest.fixture()
def element_tree(static_dir):
    return JET(from_file=os.path.join(static_dir, 'manuscript.xml'))



import xmltodict
from lxml import etree
from lxml.etree import tostring


class JatsElementTree:

    def __init__(self, from_file='', from_string=''):
        self.root = None
        if from_file:
            self.parse(from_file)
        elif from_string:
            self.fromstring(from_string)

    def parse(self, *args, **kwargs):
        """
        Loads a JATS XML file
        :param args:
        :param kwargs:
        :return: XML tree root
        """
        self.root = etree.parse(*args, **kwargs)
        return self.root

    def fromstring(self, *args, **kwargs):
        """
        Loads a JATS XML string
        :param args:
        :param kwargs:
        :return: XML tree root
        """
        self.root = etree.fromstring(*args, **kwargs)
        return self.root

    def dump(self):
        """
        Dump XML tree as string
        :return: string
        """
        return etree.tostring(self.root)

    def get_article_categories(self):
        """
        Get article categories XML representation as dict
        :return: OrderedDict
        """
        categories_tree = self.root.find('front/article-meta/article-categories')
        # return [xmltodict.parse(tostring(c)) for c in categories_tree]
        return xmltodict.parse(tostring(categories_tree))

    def get_title(self):
        """
        Get article title
        :return: string
        """
        return self.root.find('front/article-meta/title-group/article-title').text

    def get_trans_titles(self):
        """
        Get article translated titles as a "language:title" dictionary
        :return: dict
        """
        trans_titles_trees = self.root.findall('front/article-meta/title-group/trans-title-group')
        trans_titles = {}
        for trans_title_tree in trans_titles_trees:
            lang = trans_title_tree.attrib['{http://www.w3.org/XML/1998/namespace}lang']
            title_tree = trans_title_tree.find('trans-title')
            title = ''.join(title_tree.itertext())
            trans_titles[lang] = dict(title=title, **title_tree.attrib)
        return trans_titles

    def get_article_authors(self, only_persons=True):
        """
        Get article authors contribs representation as dict
        :param only_persons: Only returns contribs of type "person" (Default: True)
        :return: list
        """
        if only_persons:
            authors_contrib_group = self.root.findall(
                'front/article-meta/contrib-group[@content-type="author"]//contrib[@contrib-type="person"]')
        else:
            authors_contrib_group = self.root.findall(
                'front/article-meta/contrib-group[@content-type="author"]/contrib')
        author_contrib = []
        for authors_contrib in authors_contrib_group:
            author_contrib.append(xmltodict.parse(tostring(authors_contrib)))
        return author_contrib

    def get_article_editors(self, only_persons=True):
        """
        Get article editors contribs representation as dict
        :param only_persons: Only returns contribs of type "person" (Default: True)
        :return: list
        """
        if only_persons:
            editors_contrib_group = self.root.findall(
                'front/article-meta/contrib-group[@content-type="editor"]//contrib[@contrib-type="person"]')
        else:
            editors_contrib_group = self.root.findall(
                'front/article-meta/contrib-group[@content-type="editor"]/contrib')
        editor_contrib = []
        for editors_contrib in editors_contrib_group:
            editor_contrib.append(xmltodict.parse(tostring(editors_contrib)))
        return editor_contrib

    def get_article_affiliations(self):
        """
        Get article affiliations representation as dict
        :return: list
        """
        article_affiliations_group = self.root.findall('front/article-meta/aff')
        article_affiliations = []
        for article_affiliation in article_affiliations_group:
            parsed = xmltodict.parse(tostring(article_affiliation))
            if not(isinstance(parsed['aff']['institution'], list)):
                parsed['aff']['institution'] = [parsed['aff']['institution']]
            article_affiliations.append(parsed)
        return article_affiliations

    def get_article_publication_date(self):
        """
        Get article publication date representation as dict
        :return: OrderedDict
        """
        pubdate_root = self.root.find('front/article-meta/pub-date')
        return xmltodict.parse(tostring(pubdate_root))

    def get_volume(self):
        """
        Get article volume
        :return: string
        """
        return self.root.find('front/article-meta/volume').text

    def get_issue(self):
        """
        Get article issue
        :return: string
        """
        return self.root.find('front/article-meta/issue').text

    def get_fpage(self):
        """
        Get article begin page
        :return: string
        """
        return self.root.find('front/article-meta/fpage').text

    def get_lpage(self):
        """
        Get article end page
        :return: string
        """
        return self.root.find('front/article-meta/lpage').text

    def get_page_range(self):
        """
        Get article page range
        :return: string
        """
        return self.root.find('front/article-meta/page-range').text

    def get_article_history(self):
        """
        Get article history representation as dict
        :return: OrderedDict
        """
        history_root = self.root.find('front/article-meta/history')
        return xmltodict.parse(tostring(history_root))

    def get_article_permissions(self):
        """
        Get article permissions representation as dict
        :return: OrderedDict
        """
        permissions_root = self.root.find('front/article-meta/permissions')
        return xmltodict.parse(tostring(permissions_root))

    def get_article_abstract(self):
        """
        Get article abstract representation as dict
        :return: OrderedDict
        """
        abstract_root = self.root.find('front/article-meta/abstract')
        return xmltodict.parse(tostring(abstract_root))

    def get_article_trans_abstracts(self):
        """
        Get article translated abstracts representation as dict
        :return: list
        """
        trans_abstracts_roots = self.root.findall('front/article-meta/trans-abstract')
        trans_abstracts = []
        for trans_abstract in trans_abstracts_roots:
            trans_abstracts.append(xmltodict.parse(tostring(trans_abstract)))
        return trans_abstracts

    def get_article_keywords(self):
        """
        Get article keywords representation as dict
        :return: list
        """
        keywords_roots = self.root.findall('front/article-meta/kwd-group')
        keywords = []
        for keyword in keywords_roots:
            keywords.append(xmltodict.parse(tostring(keyword)))
        return keywords

    def get_article_funding_group(self):
        """
        Get article funding representation as dict
        :return: OrderedDict
        """
        founding_group_root = self.root.find('front/article-meta/funding-group')
        return xmltodict.parse(tostring(founding_group_root))




def test_article_categories(element_tree):
    categories = element_tree.get_article_categories()
    # assert 'en' in categories.keys()
    # assert categories['en'][0]['content-type'] == "display-channel"
    # assert categories['en'][0]['content'] == "Research Article"
    # assert categories['en'][1]['content-type'] == "heading"
    # assert categories['en'][1]['content'] == "Computational and Systems Biology"
    # assert categories['en'][2]['content-type'] == "heading"
    # assert categories['en'][2]['content'] == "Epidemiology and Global Health"
    #
    # assert 'es' in categories.keys()
    # assert categories['es'][0]['content-type'] == "display-channel"
    # assert categories['es'][0]['content'] == "Artículo de investigación"
    # assert categories['es'][1]['content-type'] == "heading"
    # assert categories['es'][1]['content'] == "Biología computacional y de sistemas"
    # assert categories['es'][2]['content-type'] == "heading"
    # assert categories['es'][2]['content'] == "Epidemiología y salud global"

    assert categories['article-categories']['subj-group'][0]['@xml:lang'] == "en"
    assert categories['article-categories']['subj-group'][0]['subject'][0]['@content-type'] == "display-channel"
    assert categories['article-categories']['subj-group'][0]['subject'][0]['#text'] == "Research Article"
    assert categories['article-categories']['subj-group'][0]['subject'][1]['@content-type'] == "heading"
    assert categories['article-categories']['subj-group'][0]['subject'][1]['#text'] == "Computational and Systems Biology"
    assert categories['article-categories']['subj-group'][0]['subject'][2]['@content-type'] == "heading"
    assert categories['article-categories']['subj-group'][0]['subject'][2]['#text'] == "Epidemiology and Global Health"

    assert categories['article-categories']['subj-group'][1]['@xml:lang'] == "es"
    assert categories['article-categories']['subj-group'][1]['subject'][0]['@content-type'] == "display-channel"
    assert categories['article-categories']['subj-group'][1]['subject'][0]['#text'] == "Artículo de investigación"
    assert categories['article-categories']['subj-group'][1]['subject'][1]['@content-type'] == "heading"
    assert categories['article-categories']['subj-group'][1]['subject'][1]['#text'] == "Biología computacional y de sistemas"
    assert categories['article-categories']['subj-group'][1]['subject'][2]['@content-type'] == "heading"
    assert categories['article-categories']['subj-group'][1]['subject'][2]['#text'] == "Epidemiología y salud global"


def test_title(element_tree):
    assert element_tree.get_title() == 'Object vision to hand action in macaque parietal, premotor, and motor cortices'


def test_trans_titles(element_tree):
    trans_titles = element_tree.get_trans_titles()
    assert 'es' in trans_titles.keys()
    assert trans_titles['es']['id'] == 'trans-title-1'
    assert trans_titles['es']['title'] == 'Objeto de visión a acción manual en cortezas parietales, premotoras y motoras de macaco'


def test_article_authors(element_tree):
    article_authors = element_tree.get_article_authors()
    assert article_authors[0]['contrib']['@contrib-type'] == "person"
    assert article_authors[0]['contrib']['@equal-contrib'] == "yes"
    assert article_authors[0]['contrib']['@corresp'] == "no"
    assert article_authors[0]['contrib']['@deceased'] == "no"
    assert article_authors[0]['contrib']['name']['surname'] == 'Schaffelhofer'
    assert article_authors[0]['contrib']['name']['given-names'] == 'Stefan'
    assert article_authors[0]['contrib']['xref'][0]['@ref-type'] == 'aff'
    assert article_authors[0]['contrib']['xref'][0]['@rid'] == 'aff1'
    assert article_authors[0]['contrib']['xref'][1]['@ref-type'] == 'aff'
    assert article_authors[0]['contrib']['xref'][1]['@rid'] == 'aff2'

    assert article_authors[1]['contrib']['@contrib-type'] == "person"
    assert article_authors[1]['contrib']['@equal-contrib'] == "no"
    assert article_authors[1]['contrib']['@corresp'] == "no"
    assert article_authors[1]['contrib']['@deceased'] == "no"
    assert article_authors[1]['contrib']['name']['surname'] == 'Scherberger'
    assert article_authors[1]['contrib']['name']['given-names'] == 'Hansjörg'
    assert article_authors[1]['contrib']['xref'][0]['@ref-type'] == 'aff'
    assert article_authors[1]['contrib']['xref'][0]['@rid'] == 'aff1'
    assert article_authors[1]['contrib']['xref'][1]['@ref-type'] == 'aff'
    assert article_authors[1]['contrib']['xref'][1]['@rid'] == 'aff3'

    assert article_authors[2]['contrib']['@contrib-type'] == "person"
    assert article_authors[2]['contrib']['@equal-contrib'] == "no"
    assert article_authors[2]['contrib']['@corresp'] == "no"
    assert article_authors[2]['contrib']['@deceased'] == "no"
    assert article_authors[2]['contrib']['name']['surname'] == 'Kelly'
    assert article_authors[2]['contrib']['name']['given-names'] == 'Laura A.'
    assert article_authors[2]['contrib']['xref']['@ref-type'] == 'aff'
    assert article_authors[2]['contrib']['xref']['@rid'] == 'aff2'

    assert article_authors[3]['contrib']['@contrib-type'] == "person"
    assert article_authors[3]['contrib']['@equal-contrib'] == "no"
    assert article_authors[3]['contrib']['@corresp'] == "no"
    assert article_authors[3]['contrib']['@deceased'] == "no"
    assert article_authors[3]['contrib']['name']['surname'] == 'Randall'
    assert article_authors[3]['contrib']['name']['given-names'] == 'Daniel Lee'
    assert article_authors[3]['contrib']['name']['suffix'] == 'Jr.'
    assert article_authors[3]['contrib']['string-name']['@content-type'] == 'alias'
    assert article_authors[3]['contrib']['string-name']['#text'] == 'Dan Randall'
    assert article_authors[3]['contrib']['xref']['@ref-type'] == 'aff'
    assert article_authors[3]['contrib']['xref']['@rid'] == 'aff3'


def test_article_editors(element_tree):
    article_authors = element_tree.get_article_editors()
    assert article_authors[0]['contrib']['@contrib-type'] == "person"
    assert article_authors[0]['contrib']['@equal-contrib'] == "no"
    assert article_authors[0]['contrib']['@corresp'] == "no"
    assert article_authors[0]['contrib']['@deceased'] == "no"
    assert article_authors[0]['contrib']['name']['surname'] == 'Kastner'
    assert article_authors[0]['contrib']['name']['given-names'] == 'Sabine'
    assert article_authors[0]['contrib']['xref']['@ref-type'] == 'aff'
    assert article_authors[0]['contrib']['xref']['@rid'] == 'aff1'


def test_article_affiliations(element_tree):
    article_affiliations = element_tree.get_article_affiliations()
    assert article_affiliations[0]['aff']['@id'] == "aff1"
    assert article_affiliations[0]['aff']['institution'][0]['@content-type'] == "orgname"
    assert article_affiliations[0]['aff']['institution'][0]['#text'] == "German Primate Center GmbH"
    assert article_affiliations[0]['aff']['institution'][1]['@content-type'] == "orgdiv1"
    assert article_affiliations[0]['aff']['institution'][1]['#text'] == "Neurobiology Laboratory"
    assert article_affiliations[0]['aff']['city'] == "Göttingen"
    assert article_affiliations[0]['aff']['country'] == "Germany"

    assert article_affiliations[1]['aff']['@id'] == "aff2"
    assert article_affiliations[1]['aff']['institution'][0]['@content-type'] == "orgname"
    assert article_affiliations[1]['aff']['institution'][0]['#text'] == "The Rockefeller University"
    assert article_affiliations[1]['aff']['institution'][1]['@content-type'] == "orgdiv1"
    assert article_affiliations[1]['aff']['institution'][1]['#text'] == "Laboratory of Neural Systems"
    assert article_affiliations[1]['aff']['city'] == "New York"
    assert article_affiliations[1]['aff']['country'] == "United States"

    assert article_affiliations[2]['aff']['@id'] == "aff3"
    assert article_affiliations[2]['aff']['institution'][0]['@content-type'] == "orgname"
    assert article_affiliations[2]['aff']['institution'][0]['#text'] == "University of Göttingen"
    assert article_affiliations[2]['aff']['institution'][1]['@content-type'] == "orgdiv1"
    assert article_affiliations[2]['aff']['institution'][1]['#text'] == "Department of Biology"
    assert article_affiliations[2]['aff']['city'] == "Göttingen"
    assert article_affiliations[2]['aff']['country'] == "Germany"

    assert article_affiliations[3]['aff']['@id'] == "aff4"
    assert article_affiliations[3]['aff']['institution'][0]['@content-type'] == "orgname"
    assert article_affiliations[3]['aff']['institution'][0]['#text'] == "Princeton University"
    assert article_affiliations[3]['aff']['city'] == "Princeton"
    assert article_affiliations[3]['aff']['country'] == "United States"


def assert_date(date_dict, date_type, date):
    assert date_dict['@date-type'] == date_type
    assert date_dict['@iso-8601-date'] == date
    assert date_dict['day'] == date.split('-')[2]
    assert date_dict['month'] == date.split('-')[1]
    assert date_dict['year'] == date.split('-')[0]


def test_article_publication_date(element_tree):
    article_pub_date = element_tree.get_article_publication_date()
    assert_date(article_pub_date['pub-date'], "pub", "1999-01-29")


def test_volume(element_tree):
    assert element_tree.get_volume() == '318'


def test_issue(element_tree):
    assert element_tree.get_issue() == '7187'


def test_fpage(element_tree):
    assert element_tree.get_fpage() == '837'


def test_lpage(element_tree):
    assert element_tree.get_lpage() == '841'


def test_page_range(element_tree):
    assert element_tree.get_page_range() == '837-841'


def test_article_history(element_tree):
    article_history = element_tree.get_article_history()
    assert_date(article_history['history']['date'][0], "accepted", "1998-06-06")
    assert_date(article_history['history']['date'][1], "received", "1998-01-05")
    assert_date(article_history['history']['date'][2], "rev-recd", "1998-05-24")
    assert_date(article_history['history']['date'][3], "rev-request", "1998-03-14")


def test_article_permissions(element_tree):
    article_permissions = element_tree.get_article_permissions()
    assert article_permissions['permissions']['copyright-statement'] == '© 2018 Substance Consortium'
    assert article_permissions['permissions']['copyright-year'] == '2018'
    assert article_permissions['permissions']['copyright-holder'] == 'Substance Consortium'
    assert article_permissions['permissions']['license']['ali:license_ref'] == 'http://creativecommons.org/licenses/by/4.0/'


def test_article_abstract(element_tree):
    article_abstract = element_tree.get_article_abstract()
    assert article_abstract['abstract']['p']['@id'] == 'p-1'


def test_article_trans_abstracts(element_tree):
    article_trans_abstracts = element_tree.get_article_trans_abstracts()
    assert article_trans_abstracts[0]['trans-abstract']['@id'] == 'trans-abstract-1'
    assert article_trans_abstracts[0]['trans-abstract']['@xml:lang'] == 'es'
    assert article_trans_abstracts[0]['trans-abstract']['p']['@id'] == 'p-1-1'


def test_article_keywords(element_tree):
    article_keywords = element_tree.get_article_keywords()
    assert article_keywords[0]['kwd-group']['@xml:lang'] == 'en'
    assert article_keywords[0]['kwd-group']['kwd'][0]['@content-type'] == 'author-keyword'
    assert article_keywords[0]['kwd-group']['kwd'][0]['#text'] == 'optogenetics'
    assert article_keywords[0]['kwd-group']['kwd'][1]['@content-type'] == 'author-keyword'
    assert article_keywords[0]['kwd-group']['kwd'][1]['#text'] == 'two-photon'
    assert article_keywords[0]['kwd-group']['kwd'][2]['@content-type'] == 'author-keyword'
    assert article_keywords[0]['kwd-group']['kwd'][2]['#text'] == 'calcium imaging'
    assert article_keywords[0]['kwd-group']['kwd'][3]['@content-type'] == 'research-organism'
    assert article_keywords[0]['kwd-group']['kwd'][3]['#text'] == 'Mouse'

    assert article_keywords[1]['kwd-group']['@xml:lang'] == 'es'
    assert article_keywords[1]['kwd-group']['kwd'][0]['@content-type'] == 'author-keyword'
    assert article_keywords[1]['kwd-group']['kwd'][0]['#text'] == 'nada'
    assert article_keywords[1]['kwd-group']['kwd'][1]['@content-type'] == 'author-keyword'
    assert article_keywords[1]['kwd-group']['kwd'][1]['#text'] == 'si'


def test_article_funding_group(element_tree):
    article_funding_group = element_tree.get_article_funding_group()
    assert article_funding_group['funding-group']['award-group']['@id'] == 'fund1'
    assert article_funding_group['funding-group']['award-group']['funding-source']['institution-wrap']['institution-id']['@institution-id-type'] == 'FundRef'
    assert article_funding_group['funding-group']['award-group']['funding-source']['institution-wrap']['institution-id']['#text'] == 'https://dx.doi.org/10.13039/100000011'
    assert article_funding_group['funding-group']['award-group']['funding-source']['institution-wrap']['institution'] == 'Howard Hughes Medical Institute'
    assert article_funding_group['funding-group']['award-group']['award-id'] == 'F32 GM089018'



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="jatsutils",
    version="0.0.1",
    author="Carlos Cesar Caballero",
    author_email="ccesar@daxslab.com",
    description="Utility for obtain JATS XML data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cccaballero/jatsutils",
    packages=setuptools.find_packages(),
    install_requires=[
        'lxml',
        'xmltodict'
    ],
    extras_require={
        'dev': [
            'tox',
            'tox-travis',
            'pytest',
            'pytest-pep8',
            'pytest-cov'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)




from jatsutils import JatsElementTree as JET
from xml.etree.ElementTree import dump
import os


def test_load_from_file(static_dir):

    test_manuscript = os.path.join(static_dir, 'manuscript.xml')

    jet = JET(from_file=test_manuscript)

    title = jet.root.find('front/article-meta/title-group/article-title').text

    assert 'Object vision to hand action in macaque parietal, premotor, and motor cortices' == title


def test_load_from_string(static_dir):

    test_manuscript = os.path.join(static_dir, 'manuscript.xml')

    data = ''
    with open(test_manuscript, 'r') as data_file:
        data = data_file.read().encode()

    jet = JET(from_string=data)

    title = jet.root.find('front/article-meta/title-group/article-title').text

    assert 'Object vision to hand action in macaque parietal, premotor, and motor cortices' == title
